#define _XOPEN_SOURCE	500

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include "cf_chunk_buffer.h"
#include "cf_log.h"
#include "cf_version.h"

Cf_chunk_buffer cf_chunk_buffer_alloc() {
	Cf_chunk_buffer ret;

	ret = (Cf_chunk_buffer) malloc(sizeof(Cf_chunk_buffer_base));

	cf_chunk_buffer_set_buf(ret, NULL);
	cf_chunk_buffer_set_buf_sz(ret, 0);
	cf_chunk_buffer_set_buf_used(ret, 0);

	return ret;
}

void cf_chunk_buffer_dealloc(Cf_chunk_buffer b) {
	int i;
	int sz = cf_chunk_buffer_get_buf_sz(b);

	for (i=0; i<sz; i++) {
		if (cf_chunk_buffer_get_buffer(b, i))
			cf_chunk_data_dealloc(cf_chunk_buffer_get_buffer(b, i));
	}
	free(cf_chunk_buffer_get_buf(b));
	free(b);
}

void cf_chunk_buffer_print(Cf_chunk_buffer b) {
	int i;
	Cf_chunk_data d;

	for (i=0; i<cf_chunk_buffer_get_buf_sz(b); i++) {
		d = cf_chunk_buffer_get_buffer(b, i);
		if (d) {
			printf("%3d [%3d]%d  ", i, cf_chunk_data_get_end(d), cf_chunk_data_get_dirty(d));
		} else {
			printf("%3d [XXX] ", i);
		}
	}

	printf("\n");
}

void cf_chunk_buffer_resize(Cf_chunk_buffer b, int sz) {
	int i;
	int osz;
	Cf_chunk_data *tmp;

	tmp = (Cf_chunk_data *) realloc(cf_chunk_buffer_get_buf(b), 
			sizeof(Cf_chunk_data)*sz);

	if (!tmp && sz) {
		cf_printf("realloc() failed in %s asked for %d bytes\n", __FUNCTION__, sz);
	} else {
		cf_chunk_buffer_set_buf(b, tmp);
	}

	osz = cf_chunk_buffer_get_buf_sz(b);

	for (i=osz; i<sz; i++) {
		cf_chunk_buffer_set_buffer(b, i, NULL);
	}

	cf_chunk_buffer_set_buf_sz(b, sz);
}

void cf_chunk_buffer_add(Cf_chunk_buffer b, int i, Cf_chunk_data d) {
	if (i >= cf_chunk_buffer_get_buf_sz(b)) {
		/* Enlarge buffer */
		cf_chunk_buffer_resize(b, i+1);
	}

	if (cf_chunk_buffer_get_buffer(b, i) != NULL) 
		cf_printf("ERROR: overwriting buffer[%d] in %s\n", i, __FUNCTION__);

	/* Store into the buffer */
	cf_chunk_buffer_set_buffer(b, i, d);

	cf_chunk_buffer_set_buf_used(b, cf_chunk_buffer_get_buf_used(b)+1);
}

void cf_chunk_buffer_del(Cf_chunk_buffer b, int i) {
	if (i >= cf_chunk_buffer_get_buf_sz(b))
		return;

	cf_chunk_buffer_set_buffer(b, i, NULL);
	cf_chunk_buffer_set_buf_used(b, cf_chunk_buffer_get_buf_used(b)-1);
}

