#ifndef __CF_CHUNK__
#define __CF_CHUNK__

#define _GNU_SOURCE

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/statfs.h>
#include <utime.h>
#include <pthread.h>

#include "cf_chunk_header.h"
#include "cf_chunk_toc.h"
#include "cf_chunk_buffer.h"

#include "cf_config.h"

typedef struct {
	Cf_chunk_header h;
	Cf_chunk_toc toc;	/* Working toc */
	Cf_chunk_toc old_toc;	/* Reference toc used to track changes */

	Cf_chunk_buffer buffer;

	int dirty;	
	int fd;			/* */
	Cf_config config;	/* */
	char compress;		/* */

	pthread_rwlock_t lock;	/* */
} Cf_chunk_file_base;

typedef Cf_chunk_file_base *Cf_chunk_file;

Cf_chunk_file cf_chunk_file_alloc();
void cf_chunk_file_dealloc(Cf_chunk_file f);

#define	cf_chunk_file_get_header(f)	(f)->h
#define	cf_chunk_file_get_toc(f)	(f)->toc
#define	cf_chunk_file_get_old_toc(f)	(f)->old_toc
#define	cf_chunk_file_get_buffer(f)	(f)->buffer
#define	cf_chunk_file_get_dirty(f)	(f)->dirty
#define	cf_chunk_file_get_fd(f)		(f)->fd
#define	cf_chunk_file_get_config(f)	(f)->config
#define	cf_chunk_file_get_compress(f)	(f)->compress
#define	cf_chunk_file_get_lock(f)	(f)->lock

#define	cf_chunk_file_set_header(f,x)		cf_chunk_file_get_header(f) = x
#define	cf_chunk_file_set_toc(f,x)		cf_chunk_file_get_toc(f) = x
#define	cf_chunk_file_set_old_toc(f,x)		cf_chunk_file_get_old_toc(f) = x
#define	cf_chunk_file_set_buffer(f,x)		cf_chunk_file_get_buffer(f) = x
#define	cf_chunk_file_set_dirty(f,x)		cf_chunk_file_get_dirty(f) = x
#define	cf_chunk_file_set_fd(f,x)		cf_chunk_file_get_fd(f) = x
#define	cf_chunk_file_set_config(f,x)		cf_chunk_file_get_config(f) = x
#define	cf_chunk_file_set_compress(f,x)		cf_chunk_file_get_compress(f) = x
#define	cf_chunk_file_set_lock(f,x)		cf_chunk_file_get_lock(f) = x

/* ---------------- */

Cf_chunk_file cf_chunk_open(int fd, Cf_config, int nocompress);
void cf_chunk_close(Cf_chunk_file f);
int cf_chunk_size(Cf_chunk_file f);

void cf_chunk_write_header(Cf_chunk_file f);

void cf_chunk_flush(Cf_chunk_file f);

/* Disable replaced by macro (below)
Cf_chunk_data cf_chunk_get_chunk(Cf_chunk_file f, int idx); 
*/
#define cf_chunk_get_chunk(f, idx)		cf_chunk_buffer_get_buffer(cf_chunk_file_get_buffer(f),(idx))

void cf_chunk_preload_chunks(Cf_chunk_file f, int start, int stop);
void cf_chunk_unload_chunks(Cf_chunk_file f);

Cf_chunk_data cf_chunk_read(Cf_chunk_file f, int idx);

/* Low level chunk reading functions */
int cf_chunk_write_raw(int fd, Cf_chunk_toc t, int idx, Cf_chunk_data d);
int cf_chunk_read_raw(int fd, Cf_chunk_toc t, int idx, Cf_chunk_data d);

/* Locking stuff */
#define cf_chunk_file_rdlock(f)			pthread_rwlock_wrlock(&cf_chunk_file_get_lock(f))
/*
#define cf_chunk_file_rdlock(f)			pthread_rwlock_rdlock(&cf_chunk_file_get_lock(f))
*/
#define cf_chunk_file_wrlock(f)			pthread_rwlock_wrlock(&cf_chunk_file_get_lock(f))
#define cf_chunk_file_unlock(f)			pthread_rwlock_unlock(&cf_chunk_file_get_lock(f))

typedef void (* Cf_writer_f)(int fd, int chunk_size, Cf_chunk_header, Cf_chunk_buffer buffer, Cf_chunk_toc, Cf_chunk_toc);

#endif
