%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cf_config_file.h"
#include "cf_log.h"

#define YYERROR_VERBOSE

Cf_config_file conf_file;
Cf_config conf;

void parser_set_config_file(Cf_config_file f) ;
void parser_set_config(Cf_config c) ;

%}

%union{
  int	int_val;
  char	str[4096];
  char  file;
  char  definition;
  char  parameter;
}
 
%start file_def

%token <int_val> NUMBER 
%token <str> TOKSTRING TOKCOMPR TOKBACKEND TOKLBRAC TOKRBRAC TOKEQUAL TOKSIZE TOKMAX TOKEXCL TOKWRITER TOKLOG TOKCOMMA TOKFORCE TOKTHRESHOLD
%type <definition> file_def mount_def
%type <parameter> name_def parameters_def  parameter_def backend_def chunk_max_def chunk_size_def compress_def ext_defs ext_def excl_def log_def writer_def scan_def threshold_def

%%

file_def :
	mount_def 
	| file_def mount_def ;

mount_def:	/* def of one mount */
	name_def parameters_def

name_def:
	TOKLBRAC TOKSTRING TOKRBRAC 
	{
#ifdef CF_DEBUG
	printf("[%s]\n", $2);
#endif

	conf = cf_config_alloc();
	cf_config_file_add(conf_file, conf);

	cf_config_set_mountpoint(conf, $2);

	} ;

parameters_def:
	parameter_def
	| parameters_def parameter_def ;

parameter_def:
	backend_def 
	| compress_def
	| chunk_size_def 
	| chunk_max_def 
	| threshold_def 
	| excl_def
	| log_def
	| scan_def
	| writer_def;


backend_def:
	TOKBACKEND TOKEQUAL TOKSTRING 
	{
#ifdef CF_DEBUG
	printf("%s = %s\n", $1, $3);
#endif

	cf_config_set_backendpath(conf, $3);

	} ;

compress_def:
	TOKCOMPR TOKEQUAL TOKSTRING
	{
#ifdef CF_DEBUG
	printf("%s = %s\n", $1, $3);
#endif

	strcpy(cf_config_get_compression_path(conf), $3);
	} ;

writer_def:
	TOKWRITER TOKEQUAL TOKSTRING
	{
#ifdef CF_DEBUG
	printf("%s = %s\n", $1, $3);
#endif

	strcpy(cf_config_get_writer_path(conf), $3);
	} ;

log_def:
	TOKLOG TOKEQUAL TOKSTRING
	{
#ifdef CF_DEBUG
	printf("%s = %s\n", $1, $3);
#endif

	strcpy(cf_config_get_logpath(conf), $3);
	} ;

excl_def:
	TOKEXCL TOKEQUAL ext_defs
	{
#ifdef CF_DEBUG
	printf("%s = ", $1);
#endif
	} ;

ext_defs:
	ext_def
	| ext_defs ext_def;


ext_def:
	TOKSTRING
	{
#ifdef CF_DEBUG
	printf("%s ", $1);
#endif
	cf_config_blacklist_add(conf, $1);
	}

chunk_size_def:
	TOKSIZE TOKEQUAL NUMBER
	{
#ifdef CF_DEBUG
	printf("%s = %d\n", $1, $3);
#endif

	cf_config_set_chunk_size(conf, $3);
	} ;


chunk_max_def:
	TOKMAX TOKEQUAL NUMBER 
	{
#ifdef CF_DEBUG
	printf("%s = %d\n", $1, $3 );
#endif

	cf_config_set_chunk_max(conf, $3);
	} ;

threshold_def:
	TOKTHRESHOLD TOKEQUAL NUMBER 
	{
#ifdef CF_DEBUG
	printf("%s = %d\n", $1, $3 );
#endif

	cf_config_set_compression_threshold(conf, $3);
	} ;


scan_def:
	TOKFORCE
	{
#ifdef CF_DEBUG
	printf("%s \n", $1 );
#endif

	cf_config_set_forced_scan(conf, 1);
	} ;


%%

void parser_set_config_file(Cf_config_file f) {
	conf_file = f;
}

void parser_set_config(Cf_config c) {
	conf = c;
}
