#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>

#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>

#include "cf_config.h"
/*
#include "cf_defines.h"
*/
#include "cf_log.h"

Cf_config cf_config_alloc() {
	Cf_config ret;

	ret = (Cf_config) malloc(sizeof(Cf_config_base));
	if (!ret)
		puts("cf_config_alloc() failed");

	/* Set some defaults */
	cf_config_set_forced_scan(ret, 0);
	cf_config_set_chunk_max(ret, 128);
	cf_config_set_chunk_ahead(ret, 1);
	cf_config_set_forced_scan(ret, 0);
	cf_config_set_chunk_size(ret, 65535);
	cf_config_set_compression_threshold(ret, 0);
	cf_config_set_next(ret, NULL);
	cf_config_set_blacklist(ret, NULL);
	cf_config_set_blacklist_size(ret, 0);

	return ret;
}

void cf_config_dealloc(Cf_config c) {
	free(c);
}

void cf_config_set_backendpath(Cf_config c, char *path) {
	strcpy(cf_config_get_backendpath(c), "");

	if (path[0] != '/') {
		/* Make relative path absolute */
		strcpy(cf_config_get_backendpath(c), getenv("PWD"));
		strcat(cf_config_get_backendpath(c), "/");
	}

	strcat(cf_config_get_backendpath(c), path);
}

void cf_config_set_mountpoint(Cf_config c, char *path) {
	strcpy(cf_config_get_mountpoint(c), path);
}

void cf_config_blacklist_add(Cf_config c, char *e) {
	char **tmp;
	char *entry;

	/* Get more mem */	
	tmp = (char **) realloc(cf_config_get_blacklist(c), (cf_config_get_blacklist_size(c)+1)* sizeof (char *));
	if (tmp) {
		cf_config_set_blacklist(c, tmp);
		cf_config_set_blacklist_size(c, cf_config_get_blacklist_size(c)+1);
	}

	entry = (char *) malloc(strlen(e)+1);
	strcpy(entry, e);

	tmp[cf_config_get_blacklist_size(c)-1] = entry;

/*	printf(" %s", entry); */
	
}

void cf_config_init(Cf_config c) {
	/* Fill in the blacklist: file extensions for files not too compress */
	int i;
	char *list[22] = { 
		"jpg",
		"jpeg",
		"gif",
		"png",
		"mov",
		"mpg",
		"mpeg",
		"avi",
		"mp3",
		"mp2",
		"rm",
		"rmvb",
		"ra",
		"ram",
		"zip",
		"gz",
		"arj",
		"lha",
		"bz2",
		"tgz",
		"tbz2",
		NULL
	};

	puts("-----");
	puts("compFUSEd configuration defaults");
	puts("Files with the following extensions will never be compressed by compFUSEd:");

	cf_config_set_blacklist(c, NULL);
	cf_config_set_blacklist_size(c, 0);

	i = 0;
	while (list[i]) {
		cf_config_blacklist_add(c, list[i]);
		i++;
	}
	puts("");


	cf_log_open(NULL);
	puts("compFUSEd configuration defaults set");
}

FILE* cf_config_open(char *path) {
	FILE *ret;
	char home[128];

	printf("Using config file ");

	/* Start with current dir */
	ret = fopen(path, "r");
	if (ret) {
		puts(path);
		return ret;
	}

	/* Look in home dir */
	strcpy(home, getenv("PWD"));
	strcat(home, path);	
	ret = fopen(home, "r");
	if (ret) {
		puts(home);
		return ret;
	}

	puts(" NONE! Problem!");

	return NULL;
}

void lowerify(char *str) {
	int i = 0;

	while(str[i] != ' ') {
		str[i] = tolower(str[i]);
		i++;
	}
}

void cf_config_warn(char *l) {
	printf("WARNING: the \"%s\" setting in your config file is not needed/supported anymore\n", l);
}

void cf_config_load_compression(Cf_config c, char *path) {
        char *error;
        void *handle = NULL;
        Cf_error_f cf_error;
        Cf_buffersizer_f cf_buffersizer;
        Cf_compressor_f cf_compressor;
        Cf_decompressor_f cf_decompressor;
        Cf_id_f cf_id;

        dlerror();
        handle = dlopen(path, RTLD_LAZY);
        if (!handle) {
                printf("ERROR: %s\n", dlerror());
        }

        dlerror();
        cf_id = (Cf_id_f) dlsym(handle, "cf_id");
        if ((error = dlerror())) {
                printf("ERROR: %s\n", error);

                exit(1);
        }

        dlerror();
        cf_compressor = (Cf_compressor_f) dlsym(handle, "cf_compressor");
        if ((error = dlerror())) {
                printf("ERROR: %s\n", error);

                exit(1);
        }


        dlerror();
        cf_decompressor = (Cf_decompressor_f) dlsym(handle, "cf_decompressor");
        if ((error = dlerror())) {
                printf("ERROR: %s\n", error);

                exit(1);
        }

        dlerror();
        cf_buffersizer = (Cf_buffersizer_f) dlsym(handle, "cf_buffersizer");
        if ((error = dlerror())) {
                printf("ERROR: %s\n", error);

                exit(1);
        }

        dlerror();
        cf_error = (Cf_error_f) dlsym(handle, "cf_error");
        if ((error = dlerror())) {
                printf("ERROR: %s\n", error);

                exit(1);
        }

        /* Install the functions */
        cf_config_set_error(c, cf_error);
        cf_config_set_buffersizer(c, cf_buffersizer);
        cf_config_set_compressor(c, cf_compressor);
        cf_config_set_decompressor(c, cf_decompressor);

        /* Set compression plugin ID */
        cf_config_set_compression_id(c, cf_id());
}

void cf_config_load_writer(Cf_config c, char *path) {
        void *whandle = NULL;

        dlerror();
        whandle = dlopen(path, RTLD_LAZY);
        if (!whandle) {
                printf("ERROR: %s\n", dlerror());
        }

        cf_config_set_writer(c, dlsym(whandle, "chunk_writer"));
}

void cf_config_open_log(Cf_config c) {
	cf_log_open(cf_config_get_logpath(c));
}

void cf_config_load_plugins(Cf_config c) {
	cf_config_load_compression(c, cf_config_get_compression_path(c));
	cf_config_load_writer(c, cf_config_get_writer_path(c));
}

int cf_config_check_backend(Cf_config c) {
        DIR *dp;

        /* Check whether backend directory does exist */
        dp = opendir(cf_config_get_backendpath(c));
        if (!dp) {
                puts("DAMN your backend directory does not exist :'( Create it first!");

                return 0;
        }
        closedir(dp);

        return 1;
}


void cf_config_print(Cf_config c) {
	printf("---------------------------------------------\n");
	printf("backend %s\n", cf_config_get_backendpath(c));	
	printf("compression %s\n", cf_config_get_compression_path(c));	
	printf("chunk writer %s\n", cf_config_get_writer_path(c));	
	printf("chunk size %d\n", cf_config_get_chunk_size(c));	
	printf("chunk max %d\n", cf_config_get_chunk_max(c));	
	printf("compression threshold %d\n", cf_config_get_compression_threshold(c));	
	printf("---------------------------------------------\n");
}
