#ifndef __CF_CONFIG__
#define __CF_CONFIG__

#include "cf_defines.h"

#include "cf_compression.h"

#define CF_BACKENDPATH_SIZE	4096

typedef struct {
	/* Compression plugins */
	Cf_error_f err;
	Cf_compressor_f compressor;
	Cf_decompressor_f decompressor;
	Cf_buffersizer_f buffersizer;
	Cf_compression compression_id;

	/* Chunk writer plugin */
	void *writer;

	unsigned int chunk_size;
	unsigned int chunk_max;
	unsigned int chunk_ahead;
	unsigned int compression_threshold;

	char mountpoint[CF_BACKENDPATH_SIZE];
	char backendpath[CF_BACKENDPATH_SIZE];
	char logpath[CF_BACKENDPATH_SIZE];
	char writer_path[CF_BACKENDPATH_SIZE];
	char compression_path[CF_BACKENDPATH_SIZE];

	char **blacklist;
	int blacklist_size;

	/* Force or not a full fs scan upon start up */
	int forced_scan;

	void* next;
} Cf_config_base;

typedef Cf_config_base *Cf_config;

Cf_config cf_config_alloc();
void cf_config_dealloc();

#define cf_config_get_error(c)			(c)->err
#define cf_config_get_compressor(c)		(c)->compressor
#define cf_config_get_decompressor(c)		(c)->decompressor
#define cf_config_get_buffersizer(c)		(c)->buffersizer
#define cf_config_get_writer(c)			(c)->writer
#define cf_config_get_compression_id(c)		(c)->compression_id
#define cf_config_get_blacklist_size(c)		(c)->blacklist_size
#define cf_config_get_blacklist(c)		(c)->blacklist
#define cf_config_get_chunk_size(c)		(c)->chunk_size
#define cf_config_get_chunk_max(c)		(c)->chunk_max
#define cf_config_get_chunk_ahead(c)		(c)->chunk_ahead
#define cf_config_get_forced_scan(c)		(c)->forced_scan
#define cf_config_get_next(c)			(c)->next
#define cf_config_get_compression_threshold(c)	(c)->compression_threshold

#define cf_config_set_error(c,f)		cf_config_get_error(c) = (f)
#define cf_config_set_compressor(c,f)		cf_config_get_compressor(c) = (f)
#define cf_config_set_decompressor(c,f)		cf_config_get_decompressor(c) = (f)
#define cf_config_set_buffersizer(c,f)		cf_config_get_buffersizer(c) = (f)
#define cf_config_set_writer(c,f)		cf_config_get_writer(c) = (f)
#define cf_config_set_compression_id(c,id)	cf_config_get_compression_id(c) = (id)
#define cf_config_set_blacklist_size(c,s)	cf_config_get_blacklist_size(c) = (s)
#define cf_config_set_blacklist(c,l)		cf_config_get_blacklist(c) = (l)
#define cf_config_set_chunk_size(c,s)		cf_config_get_chunk_size(c) = (s)
#define cf_config_set_chunk_max(c,s)		cf_config_get_chunk_max(c) = (s)
#define cf_config_set_chunk_ahead(c,s)		cf_config_get_chunk_ahead(c) = (s)
#define cf_config_set_forced_scan(c,f)		cf_config_get_forced_scan(c) = (f)
#define cf_config_set_next(c,f)		cf_config_get_next(c) = (f)
#define cf_config_set_compression_threshold(c,t)	cf_config_get_compression_threshold(c) = (t)

#define cf_config_get_logpath(c)		(c)->logpath
#define cf_config_get_backendpath(c)		(c)->backendpath
#define cf_config_get_mountpoint(c)		(c)->mountpoint
#define cf_config_get_writer_path(c)		(c)->writer_path
#define cf_config_get_compression_path(c)	(c)->compression_path

void cf_config_set_backendpath(Cf_config c, char *path);
void cf_config_set_mountpoint(Cf_config c, char *mount);

void cf_config_open_log(Cf_config c);
void cf_config_load_plugins(Cf_config c);
void cf_config_load_compression(Cf_config c, char *path);
void cf_config_load_writer(Cf_config c, char *path);
void cf_config_blacklist_add(Cf_config c, char *path);

void cf_config_init(Cf_config c);
FILE* cf_config_open(char *path);
void cf_config_read(Cf_config c, char *path);

void cf_config_print(Cf_config c);
int cf_config_check_backend(Cf_config c);

#endif
