#include "cf_defines.h"


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "cf_chunk_file.h"
#include "cf_log.h"
#include "cf_config.h"

int main(int argc, char **argv) {
	int fd;
	int sz, csz;
	float ratio;
	Cf_chunk_file f;
	Cf_chunk_header h;
	Cf_chunk_toc t;

	cf_log_open(NULL);

	struct stat st_buf;

	fd = open(argv[1], O_RDWR);

	if (fd == -1) {
		printf("File not found\n");
		exit(1);
	}

	f = cf_chunk_open(fd, NULL, 0);
	h = cf_chunk_file_get_header(f);
	t = cf_chunk_file_get_toc(f);

	fstat(fd, &st_buf);

	printf("compressed with ");
	switch (cf_chunk_header_get_coding(h)) {
		case CF_NONE:
			puts("none");
			break;

		case CF_LZO:
			puts("lzo");
			break;

		case CF_LZO2:
			puts("lzo2");
			break;

		case CF_ZLIB:
			puts("zlib");
			break;

		case CF_BZIP2:
			puts("bzip2");
			break;
		default:
			puts("UNKNOWN! Header may be corrupt");
	}

	sz = cf_chunk_size(f);
	csz = st_buf.st_size;

	ratio = ((float)(100.0 * csz)) / ((float) sz);
	printf("Compression: %d to %d (%2.1f%%)\n", sz, csz, ratio);
	printf("Compression chunk size: %d bytes\n", cf_chunk_header_get_chunk_size(h));
	printf("TOC located at %d and contains %d entries\n", cf_chunk_header_get_toc_offset(h), cf_chunk_header_get_toc_size(h));
	printf("Displaying TOC:\n");
	cf_chunk_toc_print(t);

	cf_chunk_close(f);
	close(fd);

	return 0;
}
