#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include "cf_file.h"
#include "cf_map.h"
#include "cf_log.h"

#define CF_DEBUG_OFF

Cf_map_entry cf_map_entry_alloc() {
	Cf_map_entry ret;

	ret = (Cf_map_entry) malloc(sizeof(Cf_map_entry_base));

	cf_entry_set_file(ret, NULL);
	cf_entry_set_opens(ret, 0);

	return ret;
}

Cf_map cf_map_alloc() {
	Cf_map ret;

	ret = (Cf_map) malloc(sizeof(Cf_map_base));

	if (ret) {
		memset(ret, 0, sizeof(Cf_map_base));
		cf_map_set_head(ret, NULL);
		cf_map_set_size(ret, 0);
	}

	return ret;
}

void cf_map_dealloc(Cf_map m) {
	Cf_map_entry cur;
	Cf_map_entry tmp = cf_map_get_head(m);

	/* Free the list */
	if (tmp) {
		while (tmp) {
			cur = tmp;
			tmp = (Cf_map_entry) cf_entry_get_next(tmp);
			free(cur);
		}
	}

	free(m);
}

void cf_map_init(Cf_map m, Cf_config c) {
	cf_map_set_size(m, 0);

	pthread_mutex_init(&cf_map_get_lock(m), NULL);
}

void cf_map_print(Cf_map m) {
	cf_printf("map_size %d\n", cf_map_get_size(m));
}


Cf_map_entry cf_map_findname(Cf_map m, const char *name) {
	Cf_map_entry e = cf_map_get_head(m);

	while (e) {
		if (!strcmp(cf_entry_get_name(e), name)) {
			return e;
		}
		e = (Cf_map_entry) cf_entry_get_next(e);
	}

	return NULL;
}

Cf_map_entry cf_map_findfile(Cf_map m, const Cf_chunk_file file) {
	Cf_map_entry e = cf_map_get_head(m);


	while (e) {
		if (cf_entry_get_file(e) == file) {
			return e;
		}
		e = (Cf_map_entry) cf_entry_get_next(e);
	}

	return NULL;
}


Cf_map_entry cf_map_addname(Cf_map m, const char *name, Cf_chunk_file f) {
	Cf_map_entry e;
	Cf_map_entry head = cf_map_get_head(m);

	/* Get a free entry */
	e = cf_map_findname(m, name);

	if (!e) {
		e = cf_map_entry_alloc();
		/* Add in front */
		cf_entry_set_next(e, head);
		cf_entry_set_prev(e, NULL);

		cf_map_set_size(m, cf_map_get_size(m)+1);

		if (head)
			cf_entry_set_prev(cf_map_get_head(m), e);

		cf_map_set_head(m, e);

		cf_entry_set_file(e, f);
		strcpy(cf_entry_get_name(e), name);
	}

	cf_entry_set_opens(e, cf_entry_get_opens(e) + 1);

#ifdef CF_DEBUG
	cf_printf("add %s %d\n", cf_entry_get_name(e), cf_entry_get_opens(e));
#endif

	return e;
}

/* Delete an entry from the mapping  */
int cf_map_del(Cf_map m, Cf_chunk_file f) {
	int res;
	Cf_map_entry e;
	Cf_map_entry next;
	Cf_map_entry prev;

	e = cf_map_findfile(m, f);

#ifdef CF_DEBUG
	cf_printf("del %s %d\n", cf_entry_get_name(e), cf_entry_get_opens(e));
#endif

	if (e) {
		if (cf_entry_get_opens(e) ==  1) {
			prev = cf_entry_get_prev(e);
			next = cf_entry_get_next(e);
			
			if (prev)
				cf_entry_set_next(prev, next);

			if (next)
				cf_entry_set_prev(next, prev);

			if (e == cf_map_get_head(m))
				cf_map_set_head(m, next);

			free(e);

			res = 0;
		} else {
			cf_entry_set_opens(e, cf_entry_get_opens(e) -1);
			res = -1;
		}

		cf_map_set_size(m, cf_map_get_size(m) - 1);
	} else {
		cf_printf("ERROR: not found in cf_mapping\n");
		res = -1;
	}

	return res;
}
