#ifndef __CF_MAP__
#define __CF_MAP__

#define _GNU_SOURCE


#include <sys/types.h>
#include <sys/stat.h>
#include <sys/statfs.h>
#include <utime.h>
#include <pthread.h>

#include "cf_config.h"
#include "cf_chunk_file.h"



typedef	struct {
		char name[CF_BACKENDPATH_SIZE];
		Cf_chunk_file file;
		int opens;

		void *prev;
		void *next;
} Cf_map_entry_base;

typedef Cf_map_entry_base *Cf_map_entry;

Cf_map_entry cf_map_entry_alloc();

#define	cf_entry_get_name(e)			(e)->name
#define	cf_entry_get_file(e)			(e)->file
#define	cf_entry_get_opens(e)			(e)->opens

#define	cf_entry_get_prev(e)			(e)->prev
#define	cf_entry_get_next(e)			(e)->next

#define	cf_entry_set_name(e,x)			cf_entry_get_name(e) = x
#define	cf_entry_set_file(e,x)			cf_entry_get_file(e) = x
#define	cf_entry_set_opens(e,x)			cf_entry_get_opens(e) = x

#define	cf_entry_set_prev(e,x)			cf_entry_get_prev(e) = x
#define	cf_entry_set_next(e,x)			cf_entry_get_next(e) = x

typedef struct {
	pthread_mutex_t lock;
	int map_size;
	Cf_map_entry head;
} Cf_map_base;

typedef Cf_map_base *Cf_map;

#define cf_map_get_head(m)			m->head

/*
#define cf_map_get_map(m,pos)			m->map[pos]
*/
#define cf_map_get_lock(m)			m->lock
#define cf_map_get_size(m)			m->map_size

#define cf_map_get_name(m, pos)          	cf_entry_get_name(cf_map_get_map(m,pos))
#define cf_map_get_opens(m, pos)          	cf_entry_get_opens(cf_map_get_map(m,pos))
#define cf_map_get_file(m, pos)          	cf_entry_get_file(cf_map_get_map(m,pos))

#define cf_map_set_size(m, s)			cf_map_get_size(m) = s

#define cf_map_set_name(m, pos, x)          	cf_map_get_name(m,pos) = x
#define cf_map_set_opens(m, pos, x)          	cf_map_get_opens(m,pos) = x
#define cf_map_set_file(m, pos, x)          	cf_map_get_file(m,pos) = x

#define cf_map_set_head(m,h)			cf_map_get_head(m) = (h)

Cf_map cf_map_alloc();
void cf_map_dealloc();

void cf_map_print(Cf_map );
void cf_map_init(Cf_map m, Cf_config c);

Cf_map_entry cf_map_findname(Cf_map m, const char *name);
Cf_map_entry cf_map_findfile(Cf_map m, const Cf_chunk_file file);
Cf_map_entry cf_map_addname(Cf_map m, const char *name, Cf_chunk_file f);

void cf_map_clean(Cf_map m, int pos);
int cf_map_del(Cf_map m, Cf_chunk_file f);


#endif
